/*
 * Copyright (C) 2004 Domenico Delle Side
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "getphoto.h"

#include <unistd.h>
#include <sys/stat.h>
#include <fcntl.h>

#define CREAT_MODE (S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH)
#define DIRCR_MODE ((CREAT_MODE) | S_IXUSR | S_IXGRP | S_IXOTH)


static int Open(const char *name, int type)
     /*
      * type is an hack to have an Open call that can be used both to
      * open existing files and to create new ones. If type == 0, Open
      * act on existing files, else it creates the specified file.
      *
      */
{
  int fd;
  
  if ( 0 == type )
    fd = open(name, O_RDONLY);
  else
    fd = open(name, O_CREAT | O_WRONLY, CREAT_MODE);

  if ( 0 > fd )
    exit_errno();

  return fd;
}


static ssize_t Read(int fd, void *buf, size_t count)
     /*
      * Read count bytes from fd and stores them on buf, which must be
      * an allocated buffer of needed size. If the call return -1 and
      * errno is EINTR, the process of reading is repeated. The call
      * return the number of bytes readed.
      *
      */
{
  ssize_t n_read;

 read:
  if ( 0 > (n_read = read(fd, buf, count)) )
      if ( EINTR == errno )
	goto read;

  return n_read;
}


void Mkdir(const char *dirname)
     /*
      * Create a directory named dirname, checking for errors.
      *
      */
{
  if ( 0 > mkdir(dirname, DIRCR_MODE) )
    exit_errno();
  
  change_user(dirname);
}
	

static unsigned int 
get_blksize(int filedes)
     /*
      * Stat the file pointed to by filedes and return its st_blksize,
      * the preferred size to pass to read/write.
      *
      */
{
  struct stat sb;

  if ( 0 > fstat(filedes, &sb) )
    exit_errno();

  return sb.st_blksize;
}


int check_dir(const char *path)
     /*
      * Check if path exists and is a directory.
      *
      */
{
  struct stat sb;

  if ( 0 > stat(path, &sb) )
    return -1;

  return ( (S_ISDIR(sb.st_mode) ) ? 1 : -1);
}


static ssize_t 
full_write(int fd, const void *buf, size_t count)
/*
 * Taken from coreutils-5.0.91/lib/full-write.c, with some maquillage
 *
 */
{
  size_t total = 0;
  const char *ptr = buf;
 
  while ( count > 0 )
    {
      size_t n_rw = write(fd, ptr, count);

      if ( n_rw == (size_t) -1 )
        break;

      if ( n_rw == 0 )
	break;

      total += n_rw;
      ptr += n_rw;
      count -= n_rw;
    }
  
  return total;
}


int copy_file(char *src, char *dest_dir)
     /*
      * Copy src to directory dest_dir.  We pass only the destination
      * directory, cause the file name will be the same.
      *
      */
{
  int src_fd = 0; 
  int dest_fd = 0;
  int return_value = 1;
  size_t dest_filename_size;
  unsigned int blk_size;
  char *tmp, *dest_filename;
  struct stat dest_test;

  src_fd = Open(src, 0);
  blk_size = get_blksize(src_fd);

  if ( 0 > check_dir(dest_dir) )
    Mkdir(dest_dir);

  if (NULL != (tmp = strrchr(src, '/')) )
    {
      tmp++;
      src = tmp;
    }

  /*
   * Now we must make a string with the full path of the new file,
   * including its filename. Let's do it using snprintf, it's
   * safer.
   *
   */
  
  dest_filename_size = strlen(dest_dir) + strlen(src) + (size_t)2;
  dest_filename = Calloc(dest_filename_size, sizeof(char));

  /*
   * We add 2 because we have to consider the null terminator and a
   * slash to separate dest_dir and src.
   *
   */

  snprintf(dest_filename, dest_filename_size, "%s/%s", dest_dir, src);

  /*
   * Now we can open dest_filename.
   *
   */

  if ( 0 == stat(dest_filename, &dest_test) )
    {
      print_error(GT_("File %s already exists. Exiting\n"), dest_filename);
      return_value = -1;
      goto release_all;
    }

  dest_fd = Open(dest_filename, 1);

  /*
   * Now we can start to copy the file.
   *
   */

  tmp = Calloc(blk_size, sizeof(char));

  for ( ; ; )
    {
      ssize_t n_read = Read(src_fd, tmp, blk_size);
      
      if ( 0 == n_read )
	break;

      size_t n_write = (size_t) n_read;

      if ( n_read != full_write(dest_fd, tmp, n_write) )
	{
	  print_error(GT_("Errors copying %s\n"), dest_filename);
	  unlink(dest_filename);
	  return_value = -1;
	  goto release_all;
	}
    }

  if ( 0 > change_user(dest_filename) )
    goto release_all;

 release_all:
  free(tmp);
  free(dest_filename);

   if( 2 < src_fd )
    close(src_fd);
 
  if ( 2 < dest_fd )
    close(dest_fd);

  return return_value;
}
