/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "getphoto.h"

#include <getopt.h>

#define OP_DEVICE 1
#define OP_FSTYPE 2
#define OP_MNTPNT 3
#define OP_PHOPHT 4
#define OP_DSTDIR 5
#define OP_USEDAT 6
#define OP_PRSERV 7
#define OP_RESIZE 8
#define OP_IMGWDT 9
#define OP_IMGHEI 10
#define OP_IMGQLT 11
#define OP_HELP   12




static struct option cmd_line_op[] = {
  {"device",       required_argument,  NULL,  OP_DEVICE},
  {"fs-type",      required_argument,  NULL,  OP_FSTYPE},
  {"mount-point",  required_argument,  NULL,  OP_MNTPNT},
  {"photo-path",   required_argument,  NULL,  OP_PHOPHT},
  {"dest-dir",     required_argument,  NULL,  OP_DSTDIR},
  {"usedate",      no_argument,        NULL,  OP_USEDAT},
  {"preserve",     no_argument,        NULL,  OP_PRSERV},
#ifdef HAVE_JPEGLIB_H
  {"resize",       no_argument,        NULL,  OP_RESIZE},
  {"width",        required_argument,  NULL,  OP_IMGWDT},
  {"height",       required_argument,  NULL,  OP_IMGHEI},
  {"quality",      required_argument,  NULL,  OP_IMGQLT},
#endif
  {"help",         no_argument,        NULL,  OP_HELP  },
  {(char *) 0,     no_argument,        NULL,  0        }
};




static char 
*change_option(char *option, char *new_value)
{
  free(option);
  return mystrndup(new_value, strlen(new_value));
}


static char
*get_config_filename(void)
{
  char *home, *filename;
  size_t size;

  if ( NULL == (home = getenv("HOME")) )
    print_exit(GT_("Unable to open configuration file\n"));

  /* 
   * We add 2 to get space for a '/' and the null terminator.
   */
  size = strlen(home) + strlen(CONF_FILE) + (size_t)2;
  filename = Calloc(size, sizeof(char));

  snprintf(filename, size, "%s/%s", home, CONF_FILE);
  free(home);

  return filename;
}


static 
void banner(void)
{
  printf(GT_("\n%s - Copyright (C) 2004 Domenico Delle Side\n"), PACKAGE);
  printf(GT_("Version %s\n\n"), VERSION);
  printf(GT_("%s is free software and you can use, modify and/or distribute\n"), PACKAGE);
  printf(GT_("it under the terms of the GNU General Public License.\n\n"));
}
 

static 
void usage(void)
{
  banner();
  printf(GT_("Usage:  %s  [OPTIONS]\n"), PACKAGE);
  printf(GT_("  Options are as follows:\n"));
  printf(GT_("  -s, --device        device of the digital camera\n"));
  printf(GT_("  -f, --fs-type       filesystem used on digicam's memory\n"));
  printf(GT_("  -m, --mount-point   mount point of the digital camera\n"));
  printf(GT_("  -p, --photo-path    where to find photos in the digicam\n"));
  printf(GT_("  -l, --dest-dir      where to save photos\n"));
  printf(GT_("  -u, --usedate       save photos in dest-dir/today\n"));
  printf(GT_("  -n, --preserve      don't unlink files from digicam's memory\n"));
#ifdef HAVE_JPEGLIB_H
  printf(GT_("  -r, --resize        resize photos (only for jpegs)\n"));
  printf(GT_("  -w, --width         width of resized images\n"));
  printf(GT_("  -h, --height        height of resize images\n"));
  printf(GT_("  -q, --quality       quality of resized images\n"));
#endif
  printf(GT_("  -h, --help          show this help\n\n"));
}



int main( int argc, char **argv)
{
  struct config *conf;
  char *config_filename;
  char *short_options = "s:l:";

  int c, n;

  setlocale(LC_ALL, "");
  textdomain(PACKAGE);
  bindtextdomain(PACKAGE, LOCALEDIR);

  config_filename = get_config_filename();
  conf = parse_config(config_filename);

  while ( -1 != (c = getopt_long(argc, argv, short_options, cmd_line_op, NULL)) )
    {
      switch ( c ) 
	{
	case 's':
	case OP_DEVICE:
	  conf->device = change_option(conf->device, optarg);
	  break;
	case OP_FSTYPE:
	  conf->fs_type = change_option(conf->fs_type, optarg);
	  break;
	case 'l':
	case OP_MNTPNT:
	  conf->mount_point = change_option(conf->mount_point, optarg);
	  break;
	case OP_PHOPHT:
	  conf->photo_path = change_option(conf->photo_path, optarg);
	  break;
	case OP_DSTDIR:
	  conf->dest_dir = change_option(conf->dest_dir, optarg);
	  break;
	case OP_USEDAT:
	  conf->usedate = 1;
	  break;
	case 'n':
	case OP_PRSERV:
	  conf->preserve = 1;
	  break;
#ifdef HAVE_JPEGLIB_H
	case OP_RESIZE:
	  conf->resize = 1;
	  break;
	case OP_IMGWDT:
	  conf->img.img_w = (unsigned int) atoi(optarg);
	  break;
	case OP_IMGHEI:
	  conf->img.img_h = (unsigned int) atoi(optarg);
	  break;
	case OP_IMGQLT:
	  conf->img.img_quality = (unsigned int) atoi(optarg);
#endif
	case OP_HELP:
	case '?':
	default:
	  usage();
	  exit(EXIT_FAILURE);
	}
    }

  banner();
  
  n = scan_camera(conf);

  if ( 0 == n )
    {
      print_exit(GT_("No file moved\n"));
      goto end;
    }
  else if ( 0 > n )
    {
      free_options(conf);
      exit(EXIT_FAILURE);
    }
  
  printf(GT_("\nMoved %i files\n\n"), n);
  printf(GT_("Now you can safely disconnect your digital camera.\n\n"));

#ifdef HAVE_JPEGLIB_H
  if ( conf->resize )
    {
      printf("Resizing images...\n");
      make_resized_images(conf);
    }
#endif

 end:
  free_options(conf);

  exit(EXIT_SUCCESS);
}
	  
