/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "getphoto.h"
#include <dirent.h>
#include <jpeglib.h>

/*
 * This part of getphoto is highly experimental. Functions are in
 * embrional state and error checking is near to zero. Use resizing at
 * your own risk!
 *
 */

static unsigned char * 
load_jpeg(const char *filename, unsigned int *width, 
	  unsigned int *height, unsigned int *components)
{
  struct jpeg_decompress_struct cinfo;
  struct jpeg_error_mgr jerr;
  FILE *image_file;
  int row_stride, column_stride, n = 0;
  unsigned char *buf = NULL;
  JSAMPARRAY buffer;

  if ( NULL == (image_file = fopen(filename, "rb")) )
    exit_errno();
  
  cinfo.err = jpeg_std_error(&jerr);
  jpeg_create_decompress(&cinfo);
  jpeg_stdio_src(&cinfo, image_file);
  (void) jpeg_read_header(&cinfo, TRUE);
  (void) jpeg_start_decompress(&cinfo);

  /* setting some parameters */
  *width = cinfo.image_width;
  *height = cinfo.image_height;
  *components = cinfo.output_components;
  row_stride = cinfo.output_width * *components;
  column_stride = cinfo.output_height;

  buf = calloc(row_stride * column_stride, sizeof(unsigned char));
  buffer = (*cinfo.mem->alloc_sarray)
    ((j_common_ptr) & cinfo, JPOOL_IMAGE, row_stride, 1);

  while (cinfo.output_scanline < cinfo.output_height)
    {
      (void) jpeg_read_scanlines(&cinfo, buffer, 1);
      memcpy(&buf[n * row_stride], buffer[0], row_stride);
      n++;
    }

  (void) jpeg_finish_decompress(&cinfo);
  jpeg_destroy_decompress(&cinfo);
  fclose(image_file);
    
  return buf;
}


static void
write_jpeg(const void *data, const char *filename, 
	   const unsigned int width, const unsigned int height,
	   const unsigned int components, const unsigned int quality)
{
  struct jpeg_compress_struct cinfo;
  struct jpeg_error_mgr jerr;
  FILE *image_file;
  unsigned int row_stride;
  JSAMPROW row_pointer[1]; /* pointer to JSAMPLE row[s] */
  JSAMPLE *image_buffer;

  if ( NULL == (image_file = fopen(filename, "wb")) )
    exit_errno();

  image_buffer = (JSAMPLE *)data;
  cinfo.err = jpeg_std_error(&jerr);
  
  jpeg_create_compress(&cinfo);
  jpeg_stdio_dest(&cinfo, image_file);

  /* Setting some parameters */
  cinfo.image_width = width;
  cinfo.image_height = height;
  cinfo.input_components = components;
  cinfo.in_color_space = ( ( 1 == components ) ? JCS_GRAYSCALE : JCS_RGB );
  jpeg_set_defaults(&cinfo);
  jpeg_set_quality(&cinfo, quality, TRUE);
  row_stride = width * components;

  jpeg_start_compress(&cinfo, TRUE);

  while (cinfo.next_scanline < cinfo.image_height) 
    {
      row_pointer[0] = &image_buffer[cinfo.next_scanline * row_stride];
      (void) jpeg_write_scanlines(&cinfo, row_pointer, 1);
    }  

  (void) jpeg_finish_compress(&cinfo);
  fclose(image_file);
  jpeg_destroy_compress(&cinfo);
}


/*
From: "Ivan Vecerina" <ivec@myrealbox.REMOVE.com>
Newsgroups: comp.graphics.algorithms
References: <baliu8$jmn$1@newsg4.svr.pol.co.uk>
Subject: Re: Image resizing algorithm
Date: Sat, 24 May 2003 09:17:46 +0200
Lines: 71
X-Priority: 3
X-MSMail-Priority: Normal
X-Newsreader: Microsoft Outlook Express 6.00.2800.1158
X-MimeOLE: Produced By Microsoft MimeOLE V6.00.2800.1165
NNTP-Posting-Host: 80.238.29.97
Message-ID: <3ecf1c1b$1@news.swissonline.ch>
X-Trace: news.swissonline.ch 1053760539 80.238.29.97 (24 May 2003 09:15:39 +0200)
X-Complaints-To: abuse@swissonline.ch


"Fabien Senlanne" <fabien@oneoverz.ltd.uk> wrote in message
news:baliu8$jmn$1@newsg4.svr.pol.co.uk...
> I am looking to resize an image.
> My image is currently stored in a char* buffer.
NB: for portability, you should use unsigned char*
    ( char may be signed on some platforms )

> I would like to resize from 320*200 to 512*256 in order to have a nice
> texture on my objects.
Note that OpenGL extensions that allow non-power-of-2 dimensions
are relatively commonly supported nowadays.

> I know I could do it using the stretch function provided within VC++, but
 I
> would like to have something a bit better that I could use with more
> flexibility.
>
> Basically, I need to work with 2 char* buffers and resize between both

Here's a simple C implementation of bilinear interpolation of RGB/BGR
images:
*/
void scaleImage( unsigned char const* const src, unsigned char* const dst
               , unsigned const srcWidth, unsigned const srcHeight
               , unsigned const dstWidth, unsigned const dstHeight
               )
{
 unsigned i,j,c;

 float const xScale = (dstWidth -1)*1.f/(srcWidth -1);
 float const yScale = (dstHeight-1)*1.f/(srcHeight-1);

 unsigned char* dstScan = dst;

 for( j = 0; j < dstHeight; j++)
 {
   unsigned const y1 = (unsigned/*floor*/)(j / yScale);
   unsigned const y2 = (y1 < srcHeight - 1)? y1 + 1: y1;
   float const yRat2 = j / yScale - y1;
   float const yRat1 = 1.0 - yRat2;

   /* for each pixel in a destination row */
   for( i = 0; i < dstWidth; i++)
   {
      unsigned const x1 = (unsigned/*floor*/)(i / xScale);
      unsigned const x2 = (x1 < srcWidth - 1)? x1 + 1: x1;
      float const xRat2 = i / xScale - x1;
      float const xRat1 = 1.0 - xRat2;

      unsigned char const* p11 = src + 3 * (y1 * srcWidth + x1);
      unsigned char const* p21 = src + 3 * (y1 * srcWidth + x2);
      unsigned char const* p12 = src + 3 * (y2 * srcWidth + x1);
      unsigned char const* p22 = src + 3 * (y2 * srcWidth + x2);

      /* for each rgb component */
      for( c = 0 ; c != 3 ; ++c )
        *dstScan++ = (unsigned char)
            (  yRat1 * ( xRat1 * *p11++ + xRat2 * *p21++ )
            +  yRat2 * ( xRat1 * *p12++ + xRat2 * *p22++ ) );
   }
 }
}



int resize_image(const char *source_file, const char *dest_dir, 
		 struct image img)
     /*
      * Resize source_file to (img->img_w, img->img_h) and save it in
      * dest_dir, with the same name. If one between img->img_w or
      * img->img_h is zero (not both), the corresponding value is
      * computed in order to mantain the aspect ratio of source_file.
      *
      */
{
  unsigned char *img_in, *img_out = NULL;
  char *dest_file_name;
  size_t filename_len;
  unsigned int src_w, src_h, components;

  if ( (0 == img.img_w) &&
       (0 == img.img_h) )
     {
      print_error(GT_("Please, provide width and/or height if you want to resize images.\n"));
      return -1;
     }

  img_in = load_jpeg(source_file, &src_w, &src_h, &components);
  
  if ( (0 == img.img_w) )
    img.img_w = (src_w * img.img_h) / src_h;
  
  if ( (0 == img.img_h) )
    img.img_h = (src_h * img.img_w) / src_w;

  img_out = Calloc((size_t)(img.img_w * img.img_h * components), sizeof(unsigned char));

  scaleImage(img_in, img_out, src_w, src_h, img.img_w, img.img_h);

  filename_len = strlen(source_file) + strlen(dest_dir) + (size_t)2;
  dest_file_name = Calloc(filename_len, sizeof(char));
  snprintf(dest_file_name, filename_len, "%s/%s", 
	   dest_dir, source_file);

  write_jpeg(img_out, dest_file_name, img.img_w, 
	     img.img_h, components, img.img_quality);

  change_user(dest_file_name);
  
  return 1;
}


int make_resized_images(struct config *conf)
{
  DIR *photo_dir = NULL;
  DIR *saved_dir = NULL;
  struct dirent *photo_item;
  int return_value = 1;
  char *resized_path;
  size_t res_path_len;

  if ( NULL == (saved_dir = Opendir(".")) )
    return_value = -1;

  printf(GT_("Entering to: %s\n"), conf->dest_dir);

  Chdir(conf->dest_dir);

  if ( NULL == (photo_dir = Opendir(conf->dest_dir)) )
    return_value = -1;

  res_path_len = strlen(conf->dest_dir) + strlen(RESIZED_PATH) + (size_t)2;
  resized_path = Calloc(res_path_len, sizeof(char));
  snprintf(resized_path, res_path_len, "%s/%s", 
	   conf->dest_dir, RESIZED_PATH);

  printf(GT_("Saving resized to: %s\n"), resized_path);

  if ( 0 > check_dir(resized_path) )
    Mkdir(resized_path);
 
  while ( NULL != (photo_item = readdir(photo_dir)) )
    {
      if ( ('.' == *photo_item->d_name) ||
	   ( 0 < check_dir(photo_item->d_name)) )
	continue;

      printf(GT_("Working on  %s ... "), photo_item->d_name);

      if ( 0 > resize_image(photo_item->d_name,
			    resized_path,
			    conf->img) )
	{
	  print_error(GT_("Failed to resize %s\n"), photo_item->d_name);
	  return_value = -1;
	}
      else
	printf(GT_("done.\n"));

    }

  closedir(photo_dir);
  free(resized_path);
  my_Fchdir(saved_dir);
  closedir(saved_dir);

  return return_value;
}
